@@ -15,13 +15,13 @@ namespace galera
 struct KeyData
 {
     const wsrep_buf_t* const parts;
-    long const               parts_num;
+    int  const               parts_num;
     int  const               proto_ver;
     wsrep_key_type_t const   type;
     bool const               copy;
 
     KeyData (int const pv, const wsrep_buf_t* const k,
-             long const kn, wsrep_key_type_t const tp, bool const cp)
+             int const kn, wsrep_key_type_t const tp, bool const cp)
         : parts     (k),
           parts_num (kn),
           proto_ver (pv),
@@ -29,6 +29,19 @@ struct KeyData
           copy      (cp)
     {}
 
+    static wsrep_key_type_t const BRANCH_KEY_TYPE = WSREP_KEY_SHARED;
+
+    /* Zero-level key constructor */
+    explicit
+    KeyData (int              const pv,
+             wsrep_key_type_t const tp = BRANCH_KEY_TYPE)
+        : parts (&zero_part),
+          parts_num (1),
+          proto_ver(pv),
+          type (tp),
+          copy(true)
+    {}
+
     KeyData (const KeyData& kd)
     : parts    (kd.parts),
       parts_num(kd.parts_num),
@@ -45,6 +58,9 @@ private:
 
     KeyData& operator = (const KeyData&);
 
+    /* an arbitrary constant for "zero"-level, server-wide key */
+    static wsrep_buf_t const zero_part;
+
 }; /* struct KeyData */
 
 inline std::ostream&
