@@ -38,7 +38,7 @@ KeySet::version (const std::string& ver)
     gu_throw_error(EINVAL) << "Unsupported KeySet version: " << ver; throw;
 }
 
-static const char* type_str[3] = { "SH", "SS", "EX" };
+static const char* type_str[4] = { "SH", "RE", "UP", "EX" };
 
 const char*
 KeySet::type(wsrep_key_type_t t)
@@ -249,7 +249,7 @@ KeySetOut::KeyPart::KeyPart (KeyParts&      added,
 
     /* only leaf part of the key can be not WSREP_KEY_SHARED */
     bool const leaf (part_num + 1 == kd.parts_num);
-    wsrep_key_type_t const type (leaf ? kd.type : WSREP_KEY_SHARED);
+    wsrep_key_type_t const type (leaf ? kd.type : KeyData::BRANCH_KEY_TYPE);
     int const prefix (KeySet::KeyPart::prefix(type, ws_ver));
 
     assert (kd.parts_num > part_num);
@@ -355,6 +355,8 @@ KeySetOut::append (const KeyData& kd)
 {
     int i(0);
 
+//    log_info << "Appending key data:" << kd;
+
 #ifdef CHECK_PREVIOUS_KEY
     /* find common ancestor with the previous key */
     for (;
@@ -440,17 +442,16 @@ KeySetOut::append (const KeyData& kd)
                 tmp = kp; // <- updating parent for next iteration
 #endif /* CHECK_PREVIOUS_KEY */
 
-
 //            log_info << "pushed " << kp;
         }
         catch (KeyPart::DUPLICATE& e)
         {
             assert (i + 1 == kd.parts_num);
-            /* There is a very small probability that child part thows DUPLICATE
+            /* There is a very small probability that child part throws DUPLICATE
              * even after parent was added as a new key. It does not matter:
              * a duplicate will be a duplicate in certification as well. */
 #ifndef NDEBUG
-            log_debug << "Returning after catching a DUPLICATE. Part: " << i;
+//            log_debug << "Returning after catching a DUPLICATE. Part: " << i;
 #endif /* NDEBUG */
             goto out;
         }
