@@ -15,7 +15,7 @@ std::string const Replicator::Param::signal = "signal";
 
 void Replicator::register_params(gu::Config& conf)
 {
-    conf.add(Param::debug_log, "no");
+    conf.add(Param::debug_log, "no", gu::Config::Flag::type_bool);
 #ifdef GU_DBUG_ON
     conf.add(Param::dbug, "");
     conf.add(Param::signal, "");
@@ -25,5 +25,8 @@ void Replicator::register_params(gu::Config& conf)
 const char* const
 Replicator::TRIVIAL_SST(WSREP_STATE_TRANSFER_TRIVIAL);
 
+const char* const
+Replicator::NO_SST(WSREP_STATE_TRANSFER_NONE);
+
 } /* namespace galera */
 
