@@ -94,10 +94,13 @@ typedef enum status_vars
     STATS_FC_SSENT,
 //    STATS_FC_CSENT,
     STATS_FC_RECEIVED,
+    STATS_FC_ACTIVE,
+    STATS_FC_REQUESTED,
     STATS_CERT_DEPS_DISTANCE,
     STATS_APPLY_OOOE,
     STATS_APPLY_OOOL,
     STATS_APPLY_WINDOW,
+    STATS_APPLY_WAITS,
     STATS_COMMIT_OOOE,
     STATS_COMMIT_OOOL,
     STATS_COMMIT_WINDOW,
@@ -142,10 +145,13 @@ static const struct wsrep_stats_var wsrep_stats[STATS_MAX + 1] =
     { "flow_control_sent",        WSREP_VAR_INT64,  { 0 }  },
 //    { "flow_control_conts_sent",  WSREP_VAR_INT64,  { 0 }  },
     { "flow_control_recv",        WSREP_VAR_INT64,  { 0 }  },
+    { "flow_control_active",      WSREP_VAR_STRING, { 0 }  },
+    { "flow_control_requested",   WSREP_VAR_STRING, { 0 }  },
     { "cert_deps_distance",       WSREP_VAR_DOUBLE, { 0 }  },
     { "apply_oooe",               WSREP_VAR_DOUBLE, { 0 }  },
     { "apply_oool",               WSREP_VAR_DOUBLE, { 0 }  },
     { "apply_window",             WSREP_VAR_DOUBLE, { 0 }  },
+    { "apply_waits",              WSREP_VAR_INT64,  { 0 }  },
     { "commit_oooe",              WSREP_VAR_DOUBLE, { 0 }  },
     { "commit_oool",              WSREP_VAR_DOUBLE, { 0 }  },
     { "commit_window",            WSREP_VAR_DOUBLE, { 0 }  },
@@ -215,6 +221,10 @@ galera::ReplicatorSMM::stats_get() const
     sv[STATS_FC_SSENT            ].value._int64  = stats.fc_ssent;
 //    sv[STATS_FC_CSENT            ].value._int64  = stats.fc_csent;
     sv[STATS_FC_RECEIVED         ].value._int64  = stats.fc_received;
+    sv[STATS_FC_ACTIVE           ].value._string = stats.fc_active    ?
+        "true" : "false";
+    sv[STATS_FC_REQUESTED        ].value._string = stats.fc_requested ?
+        "true" : "false";
 
     double avg_cert_interval(0);
     double avg_deps_dist(0);
@@ -223,18 +233,19 @@ galera::ReplicatorSMM::stats_get() const
 
     sv[STATS_CERT_DEPS_DISTANCE  ].value._double = avg_deps_dist;
     sv[STATS_CERT_INTERVAL       ].value._double = avg_cert_interval;
-    sv[STATS_CERT_INDEX_SIZE     ].value._int64 = index_size;
+    sv[STATS_CERT_INDEX_SIZE     ].value._int64  = index_size;
 
     double oooe;
     double oool;
     double win;
-    apply_monitor_.get_stats(&oooe, &oool, &win);
+    long long waits;
+    apply_monitor_.get_stats(&oooe, &oool, &win, &waits);
 
     sv[STATS_APPLY_OOOE          ].value._double = oooe;
     sv[STATS_APPLY_OOOL          ].value._double = oool;
     sv[STATS_APPLY_WINDOW        ].value._double = win;
-
-    commit_monitor_.get_stats(&oooe, &oool, &win);
+    sv[STATS_APPLY_WAITS         ].value._int64 = waits;
+    commit_monitor_.get_stats(&oooe, &oool, &win, &waits);
 
     sv[STATS_COMMIT_OOOE         ].value._double = oooe;
     sv[STATS_COMMIT_OOOL         ].value._double = oool;
