@@ -1,4 +1,4 @@
-/* Copyright (C) 2013-2018 Codership Oy <info@codership.com>
+/* Copyright (C) 2013-2020 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -11,6 +11,7 @@
 #include "gu_uuid.h"
 #include "gu_logger.hpp"
 #include "gu_hexdump.hpp"
+#include "gu_inttypes.hpp"
 
 #include <check.h>
 
@@ -30,12 +31,12 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
     wsrep_trx_id_t trx_id(1);
     WriteSetOut wso (dir, trx_id, KeySet::FLAT8A, 0, 0, flag1, rsv, wsv);
 
-    fail_unless (wso.is_empty());
+    ck_assert(wso.is_empty());
 
     // keep WSREP_KEY_SHARED here, see loop below
     TestKey tk0(KeySet::MAX_VERSION, WSREP_KEY_SHARED, true, "a0");
     wso.append_key(tk0());
-    fail_if (wso.is_empty());
+    ck_assert(wso.is_empty() == false);
 
     uint64_t const data_out_volatile(0xaabbccdd);
     uint32_t const data_out_persistent(0xffeeddcc);
@@ -55,7 +56,7 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
     size_t const out_size(wso.gather(source, conn, trx, out));
 
     log_info << "Gather size: " << out_size << ", buf count: " << out->size();
-    fail_if((out_size % alignment) != 0);
+    ck_assert((out_size % alignment) == 0);
 
     wsrep_seqno_t const last_seen(1);
     wsrep_seqno_t const seqno(2);
@@ -72,7 +73,7 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
         in.insert (in.end(), ptr, ptr + out[i].size);
     }
 
-    fail_if (in.size() != out_size);
+    ck_assert(in.size() == out_size);
 
     gu::Buf const in_buf = { in.data(), static_cast<ssize_t>(in.size()) };
 
@@ -86,15 +87,16 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
         mark_point();
         wsi.verify_checksum();
         wsrep_seqno_t const ls(wsi.last_seen());
-        fail_if (ls != last_seen, "Found last seen: %lld, expected: %lld",
-                 ls, last_seen);
-        fail_if (wsi.flags() != flags);
-        fail_if (0 == wsi.timestamp());
-        fail_if (wsi.annotated());
+        ck_assert_msg(ls == last_seen,
+                      "Found last seen: %" PRId64 ", expected: %" PRId64,
+                      ls, last_seen);
+        ck_assert(wsi.flags() == flags);
+        ck_assert(0 != wsi.timestamp());
+        ck_assert(wsi.annotated() == false);
 
         mark_point();
         const KeySetIn& ksi(wsi.keyset());
-        fail_if (ksi.count() != 1);
+        ck_assert(ksi.count() == 1);
 
         mark_point();
         int shared(0);
@@ -103,29 +105,29 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
             KeySet::KeyPart kp(ksi.next());
             shared += (kp.prefix() == P_SHARED);
         }
-        fail_unless(shared > 0);
+        ck_assert(shared > 0);
 
         wsi.verify_checksum();
 
         wsi.set_seqno (seqno, pa_range);
-        fail_unless(wsi.pa_range() == pa_range,
-                    "wsi.pa_range = %lld\n    pa_range = %lld",
-                    wsi.pa_range(), pa_range);
-        fail_unless(wsi.certified());
+        ck_assert_msg(wsi.pa_range() == pa_range,
+                      "wsi.pa_range = %d\n    pa_range = %d",
+                      wsi.pa_range(), pa_range);
+        ck_assert(wsi.certified());
     }
     /* repeat reading buffer after "certification" */
     {
         WriteSetIn wsi(in_buf);
         mark_point();
         wsi.verify_checksum();
-        fail_unless(wsi.certified());
-        fail_if (wsi.seqno() != seqno);
-        fail_if (wsi.flags() != (flags | WriteSetNG::F_CERTIFIED));
-        fail_if (0 == wsi.timestamp());
+        ck_assert(wsi.certified());
+        ck_assert(wsi.seqno() == seqno);
+        ck_assert(wsi.flags() == (flags | WriteSetNG::F_CERTIFIED));
+        ck_assert(0 != wsi.timestamp());
 
         mark_point();
         const KeySetIn& ksi(wsi.keyset());
-        fail_if (ksi.count() != 1);
+        ck_assert(ksi.count() == 1);
 
         mark_point();
         int shared(0);
@@ -134,30 +136,30 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
             KeySet::KeyPart kp(ksi.next());
             shared += (kp.prefix() == P_SHARED);
         }
-        fail_unless(shared > 0);
+        ck_assert(shared > 0);
 
         wsi.verify_checksum();
 
         mark_point();
         const DataSetIn& dsi(wsi.dataset());
-        fail_if (dsi.count() != 1);
+        ck_assert(dsi.count() == 1);
 
         mark_point();
         gu::Buf const d(dsi.next());
-        fail_if (d.size !=
+        ck_assert(d.size ==
                  sizeof(data_out_volatile) + sizeof(data_out_persistent));
 
         const char* dptr = static_cast<const char*>(d.ptr);
-        fail_if (*(reinterpret_cast<const uint64_t*>(dptr)) !=
+        ck_assert(*(reinterpret_cast<const uint64_t*>(dptr)) ==
                  data_out_volatile);
-        fail_if (*(reinterpret_cast<const uint32_t*>
-                                    (dptr + sizeof(data_out_volatile))) !=
+        ck_assert(*(reinterpret_cast<const uint32_t*>
+                                    (dptr + sizeof(data_out_volatile))) ==
                  data_out_persistent);
 
         mark_point();
         const DataSetIn& usi(wsi.unrdset());
-        fail_if (usi.count() != 0);
-        fail_if (usi.size()  != 0);
+        ck_assert(usi.count() == 0);
+        ck_assert(usi.size()  == 0);
     }
 
     mark_point();
@@ -167,16 +169,16 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
         WriteSetIn wsi(in_buf);
         mark_point();
         wsi.verify_checksum();
-        fail_unless(wsi.certified());
-        fail_if (wsi.pa_range()   != pa_range);
-        fail_if (wsi.seqno()      != seqno);
-        fail_if (memcmp(&wsi.source_id(), &source, sizeof(source)));
-        fail_if (wsi.conn_id()    != conn);
-        fail_if (wsi.trx_id()     != trx);
+        ck_assert(wsi.certified());
+        ck_assert(wsi.pa_range()   == pa_range);
+        ck_assert(wsi.seqno()      == seqno);
+        ck_assert(!memcmp(&wsi.source_id(), &source, sizeof(source)));
+        ck_assert(wsi.conn_id()    == conn);
+        ck_assert(wsi.trx_id()     == trx);
     }
     catch (gu::Exception& e)
     {
-        fail_if (e.get_errno() != EINVAL);
+        ck_assert(e.get_errno() == EINVAL);
     }
 
     mark_point();
@@ -209,16 +211,16 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
         wsi.read_buf(tmp_buf); // next  - initialize from buffer
         mark_point();
         wsi.verify_checksum();
-        fail_unless(wsi.certified());
-        fail_if (wsi.pa_range()        != pa_range);
-        fail_if (wsi.seqno()           != seqno);
-        fail_if (wsi.keyset().count()  != 0);
-        fail_if (wsi.dataset().count() == 0);
-        fail_if (wsi.unrdset().count() != 0);
+        ck_assert(wsi.certified());
+        ck_assert(wsi.pa_range()        == pa_range);
+        ck_assert(wsi.seqno()           == seqno);
+        ck_assert(wsi.keyset().count()  == 0);
+        ck_assert(wsi.dataset().count() != 0);
+        ck_assert(wsi.unrdset().count() == 0);
     }
     catch (gu::Exception& e)
     {
-        fail_if (e.get_errno() != EINVAL);
+        ck_assert(e.get_errno() == EINVAL);
     }
 
     in[in.size() - 1] ^= 1; // corrupted the last byte (payload)
@@ -230,11 +232,11 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
         WriteSetIn wsi(in_buf);
         mark_point();
         wsi.verify_checksum();
-        fail("payload corruption slipped through 1");
+        ck_abort_msg("payload corruption slipped through 1");
     }
     catch (gu::Exception& e)
     {
-        fail_if (e.get_errno() != EINVAL);
+        ck_assert(e.get_errno() == EINVAL);
     }
 
     try /* this is to test background checksumming + corruption */
@@ -245,16 +247,16 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
 
         try {
             wsi.verify_checksum();
-            fail("payload corruption slipped through 2");
+            ck_abort_msg("payload corruption slipped through 2");
         }
         catch (gu::Exception& e)
         {
-            fail_if (e.get_errno() != EINVAL);
+            ck_assert(e.get_errno() == EINVAL);
         }
     }
     catch (std::exception& e)
     {
-        fail("%s", e.what());
+        ck_abort_msg("%s", e.what());
     }
 
     in[2] ^= 1; // corrupted 3rd byte of header
@@ -263,19 +265,21 @@ static void ver3_basic(gu::RecordSet::Version const rsv,
     {
         WriteSetIn wsi(in_buf, 2 /* this should postpone payload checksum */);
         wsi.verify_checksum();
-        fail("header corruption slipped through");
+        ck_abort_msg("header corruption slipped through");
     }
     catch (gu::Exception& e)
     {
-        fail_if (e.get_errno() != EINVAL);
+        ck_assert(e.get_errno() == EINVAL);
     }
 }
 
+#ifndef GALERA_ONLY_ALIGNED
 START_TEST (ver3_basic_rsv1)
 {
     ver3_basic(gu::RecordSet::VER1, WriteSetNG::VER3);
 }
 END_TEST
+#endif /* GALERA_ONLY_ALIGNED */
 
 START_TEST (ver3_basic_rsv2_wsv3)
 {
@@ -304,11 +308,11 @@ static void ver3_annotation(gu::RecordSet::Version const rsv)
     WriteSetOut wso (dir, trx_id, KeySet::FLAT16, 0, 0, flag1, rsv,
                      WriteSetNG::VER3);
 
-    fail_unless (wso.is_empty());
+    ck_assert(wso.is_empty());
 
     TestKey tk0(KeySet::MAX_VERSION, WSREP_KEY_SHARED, true, "key0");
     wso.append_key(tk0());
-    fail_if (wso.is_empty());
+    ck_assert(wso.is_empty() == false);
 
     uint64_t const data(0xaabbccdd);
     std::string const annotation("0xaabbccdd");
@@ -323,8 +327,8 @@ static void ver3_annotation(gu::RecordSet::Version const rsv)
     size_t const out_size(wso.gather(source, conn, trx, out));
 
     log_info << "Gather size: " << out_size << ", buf count: " << out->size();
-    fail_if((out_size % alignment) != 0);
-    fail_if(out_size < (sizeof(data) + annotation.size()));
+    ck_assert((out_size % alignment) == 0);
+    ck_assert(out_size >= (sizeof(data) + annotation.size()));
 
     wsrep_seqno_t const last_seen(1);
     wso.finalize(last_seen, 0);
@@ -338,7 +342,7 @@ static void ver3_annotation(gu::RecordSet::Version const rsv)
         in.insert (in.end(), ptr, ptr + out[i].size);
     }
 
-    fail_if (in.size() != out_size);
+    ck_assert(in.size() == out_size);
 
     gu::Buf const in_buf = { in.data(), static_cast<ssize_t>(in.size()) };
 
@@ -349,31 +353,34 @@ static void ver3_annotation(gu::RecordSet::Version const rsv)
     mark_point();
     wsi.verify_checksum();
     wsrep_seqno_t const ls(wsi.last_seen());
-    fail_if (ls != last_seen, "Found last seen: %lld, expected: %lld",
-             ls, last_seen);
-    fail_if (wsi.flags() != flags);
-    fail_if (0 == wsi.timestamp());
-    fail_if (!wsi.annotated());
+    ck_assert_msg(ls == last_seen,
+                  "Found last seen: %" PRId64 ", expected: %" PRId64,
+                  ls, last_seen);
+    ck_assert(wsi.flags() == flags);
+    ck_assert(0 != wsi.timestamp());
+    ck_assert(wsi.annotated());
 
     /* check that annotation has survived */
     std::ostringstream os;
     wsi.write_annotation(os);
     std::string const res(os.str().c_str());
 
-    fail_if(annotation.length() != res.length(),
-            "Initial ann. length: %zu, resulting ann.length: %zu",
-            annotation.length(), res.length());
+    ck_assert_msg(annotation.length() == res.length(),
+                  "Initial ann. length: %zu, resulting ann.length: %zu",
+                  annotation.length(), res.length());
 
-    fail_if(annotation != res,
-            "Initial annotation: '%s', resulting annotation: '%s'",
-            annotation.c_str(), res.c_str());
+    ck_assert_msg(annotation == res,
+                  "Initial annotation: '%s', resulting annotation: '%s'",
+                  annotation.c_str(), res.c_str());
 }
 
+#ifndef GALERA_ONLY_ALIGNED
 START_TEST (ver3_annotation_rsv1)
 {
     ver3_annotation(gu::RecordSet::VER1);
 }
 END_TEST
+#endif /* GALERA_ONLY_ALIGNED */
 
 START_TEST (ver3_annotation_rsv2)
 {
@@ -386,14 +393,18 @@ Suite* write_set_ng_suite ()
     Suite* s = suite_create ("WriteSet");
 
     TCase* t = tcase_create ("WriteSet basic");
+#ifndef GALERA_ONLY_ALIGNED
     tcase_add_test (t, ver3_basic_rsv1);
+#endif
     tcase_add_test (t, ver3_basic_rsv2_wsv3);
     tcase_add_test (t, ver3_basic_rsv2_wsv4);
     tcase_set_timeout(t, 60);
     suite_add_tcase (s, t);
 
     t = tcase_create ("WriteSet annotation");
+#ifndef GALERA_ONLY_ALIGNED
     tcase_add_test (t, ver3_annotation_rsv1);
+#endif
     tcase_add_test (t, ver3_annotation_rsv2);
     tcase_set_timeout(t, 60);
     suite_add_tcase (s, t);
