@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2020 Codership Oy <info@codership.com>
+ * Copyright (C) 2019-2023 Codership Oy <info@codership.com>
  */
 
 
@@ -920,18 +920,285 @@ END_TEST
 
 #ifdef GALERA_HAVE_SSL
 
+#include <openssl/bn.h>
+#include <openssl/conf.h>
+#include <openssl/engine.h>
+#include <openssl/pem.h>
+#include <openssl/x509v3.h>
+#include <openssl/ssl.h>
+#include <openssl/opensslv.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <dirent.h>
 #include <signal.h>
 
+#include <vector>
+
+static std::string get_cert_dir()
+{
+    assert(::strlen(GU_ASIO_TEST_CERT_DIR) > 0);
+    const std::string ret{ GU_ASIO_TEST_CERT_DIR };
+    auto* dir = opendir(ret.c_str());
+    if (!dir)
+    {
+        if (mkdir(ret.c_str(), S_IRWXU))
+        {
+            const auto* errstr = ::strerror(errno);
+            gu_throw_fatal << "Could not create dir " << ret << ": " << errstr;
+        }
+    }
+    else
+    {
+        closedir(dir);
+    }
+    return GU_ASIO_TEST_CERT_DIR;
+}
+
+static int password_cb(char*, int, int, void*) { return 0; }
+
+static void throw_error(const char* msg)
+{
+    gu_throw_fatal << msg << ": " << ERR_error_string(ERR_get_error(), nullptr);
+}
+
+static EVP_PKEY* create_key()
+{
+#if OPENSSL_VERSION_MAJOR < 3
+    auto* bn = BN_new();
+    if (!bn)
+    {
+        throw_error("could not create BN");
+    }
+    BN_set_word(bn, 0x10001);
+    auto* rsa = RSA_new();
+    if (!rsa)
+    {
+        BN_free(bn);
+        throw_error("could not create RSA");
+    }
+    RSA_generate_key_ex(rsa, 2048, bn, nullptr);
+    auto* pkey = EVP_PKEY_new();
+    if (!pkey)
+    {
+        BN_free(bn);
+        RSA_free(rsa);
+        throw_error("could not create PKEY");
+    }
+    EVP_PKEY_set1_RSA(pkey, rsa);
+    RSA_free(rsa);
+    BN_free(bn);
+    return pkey;
+#else
+    auto* ret = EVP_RSA_gen(2048);
+    if (!ret)
+    {
+        throw_error("could not create RSA");
+    }
+    return ret;
+#endif /* OPENSSL_VERSION_MAJOR < 3 */
+}
+
+static FILE* open_file(const std::string& path, const char* mode)
+{
+    auto* ret = fopen(path.c_str(), mode);
+    if (!ret)
+    {
+        const auto* errstr = ::strerror(errno);
+        gu_throw_fatal << "Could not open file " << path << ": "
+                       << errstr;
+    }
+    return ret;
+}
+
+static void write_key(EVP_PKEY* pkey, const std::string& filename)
+{
+    const std::string cert_dir = get_cert_dir();
+    const std::string key_file_path = cert_dir + "/" + filename;
+    auto* key_file = open_file(key_file_path, "wb");
+    if (!PEM_write_PrivateKey(key_file, pkey, nullptr, nullptr, 0, password_cb,
+                              nullptr))
+    {
+        throw_error("Could not write key");
+    }
+    fclose(key_file);
+}
+
+static void set_x509v3_extensions(X509* x509, X509* issuer)
+{
+    auto* conf_bio = BIO_new(BIO_s_mem());
+    std::string ext{ "[extensions]\n"
+                     "authorityKeyIdentifier=keyid,issuer\n"
+                     "subjectKeyIdentifier=hash\n" };
+    if (!issuer)
+    {
+        ext += "basicConstraints=critical,CA:TRUE\n";
+    }
+    else
+    {
+        ext += "keyUsage=digitalSignature,keyEncipherment\n";
+        ext += "basicConstraints=CA:FALSE\n";
+    }
+    BIO_printf(conf_bio, "%s", ext.c_str());
+    auto* conf = NCONF_new(nullptr);
+    long errorline = -1;
+    int err;
+    if ((err = NCONF_load_bio(conf, conf_bio, &errorline)) <= 0)
+    {
+        gu_throw_fatal << "Could not load conf: " << err;
+    }
+    if (errorline != -1)
+    {
+        gu_throw_fatal << "Could not load conf,  errorline: " << errorline;
+    }
+    // TODO: V3 extensions
+    X509V3_CTX ctx;
+    X509V3_set_ctx(&ctx, issuer ? issuer : x509, x509, nullptr, nullptr, 0);
+    X509V3_set_nconf(&ctx, conf);
+    if (!X509V3_EXT_add_nconf(conf, &ctx, (char *)"extensions", x509))
+    {
+        throw_error("Could not add extension");
+    }
+    NCONF_free(conf);
+    BIO_free(conf_bio);
+}
+
+static X509* create_x509(EVP_PKEY* pkey, X509* issuer, const char* cn)
+{
+    auto* x509 = X509_new();
+    /* According to standard, value 2 means version 3. */
+    X509_set_version(x509, 2);
+    ASN1_INTEGER_set(X509_get_serialNumber(x509), 1);
+    X509_gmtime_adj(X509_get_notBefore(x509), 0);
+    X509_gmtime_adj(X509_get_notAfter(x509), 31536000L);
+    X509_set_pubkey(x509, pkey);
+
+    auto* name = X509_get_subject_name(x509);
+    static const unsigned char C_str [] = "FI";
+    static const unsigned char ST_str[] = "Uusimaa";
+    static const unsigned char L_str [] = "Helsinki";
+    static const unsigned char O_str [] = "Codership";
+    static const unsigned char OU_str[] = "Galera Devel";
+    X509_NAME_add_entry_by_txt(name, "C",  MBSTRING_ASC, C_str,  -1, -1, 0);
+    X509_NAME_add_entry_by_txt(name, "ST", MBSTRING_ASC, ST_str, -1, -1, 0);
+    X509_NAME_add_entry_by_txt(name, "L",  MBSTRING_ASC, L_str,  -1, -1, 0);
+    X509_NAME_add_entry_by_txt(name, "O",  MBSTRING_ASC, O_str,  -1, -1, 0);
+    X509_NAME_add_entry_by_txt(name, "OU", MBSTRING_ASC, OU_str, -1, -1, 0);
+    X509_NAME_add_entry_by_txt(name, "CN", MBSTRING_ASC,
+                               reinterpret_cast<const unsigned char*>(cn),
+                               -1, -1, 0);
+    if (!issuer)
+    {
+        /* Self signed */
+        X509_set_issuer_name(x509, name);
+    }
+    else
+    {
+        X509_set_issuer_name(x509, X509_get_subject_name(issuer));
+    }
+
+    set_x509v3_extensions(x509, issuer);
+
+    X509_sign(x509, pkey, EVP_sha256());
+
+    return x509;
+}
+
+static void write_x509(X509* x509, const std::string& filename)
+{
+    const std::string cert_dir = get_cert_dir();
+    const std::string file_path = cert_dir + "/" + filename;
+    auto* file = open_file(file_path, "wb");
+    if (!PEM_write_X509(file, x509))
+    {
+        throw_error("Could not write x509");
+    }
+    fclose(file);
+}
+
+static void write_x509_list(const std::vector<X509*>& certs,
+                            const std::string& filename)
+{
+    const std::string cert_dir = get_cert_dir();
+    const std::string file_path = cert_dir + "/" + filename;
+    auto* file = open_file(file_path, "wb");
+    for (auto* x509 : certs)
+    {
+        if (!PEM_write_X509(file, x509))
+        {
+            throw_error("Could not write x509");
+        }
+    }
+    fclose(file);
+}
+
+/* Self signed CA + certificate */
+static void generate_self_signed()
+{
+    auto* pkey = create_key();
+    write_key(pkey, "galera_key.pem");
+    auto* ca = create_x509(pkey, nullptr, "Galera Root");
+    write_x509(ca, "galera_ca.pem");
+
+    auto* cert = create_x509(pkey, ca, "Galera Cert");
+    write_x509(cert, "galera_cert.pem");
+    X509_free(cert);
+    X509_free(ca);
+    EVP_PKEY_free(pkey);
+}
+
+/*
+                             ---- Server cert 1
+                            /
+  Root CA - Intermediate CA
+                            \---- Server cert 2
+
+  Two bundles consisting of intermediate CA and server certificate
+  are created for servers 1 and 2.
+ */
+static void generate_chains()
+{
+    auto* root_ca_key = create_key();
+    auto* root_ca = create_x509(root_ca_key, nullptr, "Galera Root CA");
+    auto* int_ca_key = create_key();
+    auto* int_ca = create_x509(int_ca_key, root_ca, "Galera Intermediate CA");
+
+    auto* server_1_key = create_key();
+    auto* server_1_cert = create_x509(server_1_key, int_ca, "Galera Server 1");
+    auto* server_2_key = create_key();
+    auto* server_2_cert = create_x509(server_2_key, int_ca, "Galera Server 2");
+
+    write_x509(root_ca, "galera-ca.pem");
+    write_key(server_1_key, "galera-server-1.key");
+    write_x509_list({ server_1_cert, int_ca }, "bundle-galera-server-1.pem");
+    write_key(server_2_key, "galera-server-2.key");
+    write_x509_list({ server_2_cert, int_ca }, "bundle-galera-server-2.pem");
+
+    X509_free(server_2_cert);
+    EVP_PKEY_free(server_2_key);
+    X509_free(server_1_cert);
+    EVP_PKEY_free(server_1_key);
+    X509_free(int_ca);
+    EVP_PKEY_free(int_ca_key);
+    X509_free(root_ca);
+    EVP_PKEY_free(root_ca_key);
+}
+
+static void generate_certificates()
+{
+#if OPENSSL_VERSION_NUMBER < 0x30004000L
+#ifdef OPENSSL_INIT_LOAD_SSL_STRINGS
+    OPENSSL_init_ssl(OPENSSL_INIT_LOAD_SSL_STRINGS, NULL);
+#endif
+#endif
+
+  generate_self_signed();
+  generate_chains();
+}
+
 //
 // SSL
 //
 
-static std::string get_cert_dir()
-{
-    // This will be set by CMake/preprocessor.
-    return GU_ASIO_TEST_CERT_DIR;
-}
-
 static gu::Config get_ssl_config()
 {
     gu::Config ret;
@@ -1197,8 +1464,13 @@ START_TEST(test_ssl_certificate_chain)
     client_io_service.run_one(); // Process async connect
     server_io_service.run_one(); // Accept
     client_io_service.run_one(); // Client hello
-    server_io_service.run_one(); // Server handles
-    ck_assert(acceptor_handler->accepted_socket() != 0);
+    client_io_service.run_one(); // Client hello IO completion
+    // server_io_service.run_one(); // Server handles
+    while (acceptor_handler->accepted_socket() != 0)
+    {
+        client_io_service.poll_one();
+        server_io_service.poll_one();
+    }
 }
 END_TEST
 
@@ -1224,7 +1496,13 @@ START_TEST(test_ssl_invalid_cert)
     client_io_service.run_one(); // Process async connect
     server_io_service.run_one(); // Accept
     client_io_service.run_one(); // Client hello
-    server_io_service.run_one(); // Server handles
+    client_io_service.run_one(); // Client hello IO completion
+    // server_io_service.run_one(); // Server handles
+    while (not handler->last_error_code())
+    {
+        client_io_service.poll_one();
+        server_io_service.poll_one();
+    }
     ck_assert(not acceptor_handler->accepted_socket());
     ck_assert_msg(handler->last_error_code().message().find(
                       "unable to get local issuer certificate") !=
@@ -1371,6 +1649,14 @@ struct TlsServiceClientTestFixture
             server_io_service.run_one();
         }
     }
+
+    void run_client_while(const std::function<bool()>& pred)
+    {
+        while (pred())
+        {
+            client_io_service.run_one();
+        }
+    }
 };
 
 START_TEST(test_client_handshake_want_read)
@@ -1385,10 +1671,8 @@ START_TEST(test_client_handshake_want_read)
         cbs,
         f.acceptor_handler->accepted_handler());
     f.server_io_service.run_one();
-    while (f.client_engine->count_client_handshake_called < 2)
-    {
-        f.client_io_service.run_one();
-    }
+    f.run_client_while(
+        [&f] { return f.client_engine->count_client_handshake_called < 2; });
 }
 END_TEST
 
@@ -1396,10 +1680,8 @@ START_TEST(test_client_handshake_want_write)
 {
     TlsServiceClientTestFixture f;
     f.client_engine->next_result = gu::AsioStreamEngine::want_write;
-    while (f.client_engine->count_client_handshake_called < 2)
-    {
-        f.client_io_service.run_one();
-    }
+    f.run_client_while(
+        [&f] { return f.client_engine->count_client_handshake_called < 2; });
 }
 END_TEST
 
@@ -1408,6 +1690,7 @@ START_TEST(test_client_handshake_eof)
     TlsServiceClientTestFixture f;
     f.client_engine->next_result = gu::AsioStreamEngine::eof;
     f.client_io_service.run_one();
+    f.client_io_service.run_one(); // IO completion
     ck_assert(f.socket_handler->connect_handler_called());
     ck_assert(f.socket_handler->last_error_code().is_eof());
     ck_assert(f.client_engine->count_client_handshake_called == 1);
@@ -1421,6 +1704,7 @@ START_TEST(test_client_handshake_eof2)
     // call. The EOF will now returned in client handshake handler.
     f.client_engine->next_result = gu::AsioStreamEngine::want_write;
     f.client_io_service.run_one();
+    f.client_io_service.run_one(); // IO completion
     f.client_engine->next_result = gu::AsioStreamEngine::eof;
     f.client_io_service.run_one();
     ck_assert(f.socket_handler->connect_handler_called());
@@ -1435,6 +1719,7 @@ START_TEST(test_client_handshake_error)
     f.client_engine->next_result = gu::AsioStreamEngine::error;
     f.client_engine->next_error = EPIPE;
     f.client_io_service.run_one();
+    f.client_io_service.run_one(); // IO completion
     ck_assert(f.socket_handler->connect_handler_called());
     ck_assert(f.socket_handler->last_error_code().value() == EPIPE);
     ck_assert(f.client_engine->count_client_handshake_called == 1);
@@ -1448,6 +1733,7 @@ START_TEST(test_client_handshake_error2)
     // call. The error will now returned in client handshake handler.
     f.client_engine->next_result = gu::AsioStreamEngine::want_write;
     f.client_io_service.run_one();
+    f.client_io_service.run_one(); // IO completion
     f.client_engine->next_result = gu::AsioStreamEngine::error;
     f.client_engine->next_error = EPIPE;
     f.client_io_service.run_one();
@@ -1481,34 +1767,53 @@ struct TlsServiceServerTestFixture
         acceptor->async_accept(acceptor_handler, server_engine);
         socket->async_connect(acceptor->listen_addr(), socket_handler);
         client_io_service.run_one();
+        client_io_service.run_one(); // IO completion
         // client_io_service runs out of work. Reset to make
         // followig calls succeed
         client_io_service.reset();
     }
+
+    void complete_server_handshake()
+    {
+        server_io_service.run_one();
+        server_io_service.run_one();
+    }
+
+    void run_server_while(const std::function<bool()>& pred)
+    {
+        while (pred())
+        {
+            server_io_service.run_one();
+        }
+    }
+
+    void run_client_while(const std::function<bool()>& pred)
+    {
+        while (pred())
+        {
+            client_io_service.run_one();
+        }
+    }
 };
 
 START_TEST(test_server_handshake_want_read)
 {
     TlsServiceServerTestFixture f;
     f.server_engine->next_result = gu::AsioStreamEngine::want_read;
-    while (f.server_engine->count_server_handshake_called < 1)
-    {
-        f.server_io_service.run_one();
-    }
+    f.run_server_while(
+        [&f]() { return f.server_engine->count_server_handshake_called < 1; });
     ck_assert(f.server_engine->count_server_handshake_called == 1);
+
     // Write to connected socket to make accepted socket readable
     std::array<gu::AsioConstBuffer, 2> cbs;
     cbs[0] = gu::AsioConstBuffer("clie", 4);
     cbs[1] = gu::AsioConstBuffer();
     f.socket->async_write(cbs, f.socket_handler);
-    while (f.socket_handler->bytes_written() < 4)
-    {
-        f.client_io_service.run_one();
-    }
-    while (f.server_engine->count_server_handshake_called < 2)
-    {
-        f.server_io_service.run_one();
-    }
+
+    f.run_client_while(
+        [&f]() { return f.socket_handler->bytes_written() < 4; });
+    f.run_server_while(
+        [&f]() { return f.server_engine->count_server_handshake_called < 2; });
 }
 END_TEST
 
@@ -1516,10 +1821,8 @@ START_TEST(test_server_handshake_want_write)
 {
     TlsServiceServerTestFixture f;
     f.server_engine->next_result = gu::AsioStreamEngine::want_write;
-    while (f.server_engine->count_server_handshake_called < 2)
-    {
-        f.server_io_service.run_one();
-    }
+    f.run_server_while(
+        [&f]() { return f.server_engine->count_server_handshake_called < 2; });
 }
 END_TEST
 
@@ -1541,7 +1844,7 @@ START_TEST(test_server_handshake_eof2)
     // First op causes accept handler to restart server handshake call.
     // The EOF will now handled in server handshake handler.
     f.server_engine->next_result = gu::AsioStreamEngine::want_write;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     f.server_engine->next_result = gu::AsioStreamEngine::eof;
     f.server_io_service.run_one();
     // Acceptor silently discards accepted socket which fails during
@@ -1556,7 +1859,7 @@ START_TEST(test_server_handshake_error)
     TlsServiceServerTestFixture f;
     f.server_engine->next_result = gu::AsioStreamEngine::error;
     f.server_engine->next_error = EPIPE;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     // Acceptor silently discards accepted socket which fails during
     // handshake and restarts async accept.
     ck_assert(f.acceptor_handler->accepted_socket() == 0);
@@ -1570,7 +1873,7 @@ START_TEST(test_server_handshake_error2)
     // First op causes accept handler to restart server handshake call.
     // The error will now handled in server handshake handler.
     f.server_engine->next_result = gu::AsioStreamEngine::want_write;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     f.server_engine->next_result = gu::AsioStreamEngine::error;
     f.server_engine->next_error = EPIPE;
     f.server_io_service.run_one();
@@ -1584,7 +1887,7 @@ END_TEST
 START_TEST(test_read_want_read)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
 
     std::array<gu::AsioConstBuffer, 2> cbs;
@@ -1597,7 +1900,8 @@ START_TEST(test_read_want_read)
     f.acceptor_handler->accepted_socket()->async_read(
         gu::AsioMutableBuffer(buf.data(), buf.size()),
         f.acceptor_handler->accepted_handler());
-    f.server_io_service.run_one();
+
+    f.run_server_while([&f]() { return f.server_engine->count_read_called < 1; });
     ck_assert(f.server_engine->count_read_called == 1);
     ck_assert(f.acceptor_handler->accepted_handler()->bytes_read() == 4);
     // Write socket to make accepted socket readable, but do not start
@@ -1606,8 +1910,10 @@ START_TEST(test_read_want_read)
     f.client_io_service.reset();
     f.client_io_service.run_one();
     f.server_engine->next_result = gu::AsioStreamEngine::success;
-    f.server_io_service.run_one();
-    ck_assert(f.server_engine->count_read_called == 2);
+    const auto expect_count_read_called = f.server_engine->count_read_called + 1;
+    f.run_server_while(
+		       [&f, expect_count_read_called]() { return f.server_engine->count_read_called < expect_count_read_called; });
+    ck_assert(f.server_engine->count_read_called == expect_count_read_called);
     // Extra read should just call read() but the communication should
     // be internal, the handler should not see received data.
     ck_assert(f.acceptor_handler->accepted_handler()->bytes_read() == 4);
@@ -1617,7 +1923,7 @@ END_TEST
 START_TEST(test_read_want_write)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
 
     std::array<gu::AsioConstBuffer, 2> cbs;
@@ -1630,37 +1936,43 @@ START_TEST(test_read_want_write)
     f.acceptor_handler->accepted_socket()->async_read(
         gu::AsioMutableBuffer(buf.data(), buf.size()),
         f.acceptor_handler->accepted_handler());
-    f.server_io_service.run_one();
-    ck_assert(f.server_engine->count_read_called == 1);
+    const auto expect_count_read_called = f.server_engine->count_read_called + 1;
+    f.run_server_while(
+		       [&f, expect_count_read_called]() { return f.server_engine->count_read_called < expect_count_read_called; });
+
+    ck_assert(f.server_engine->count_read_called == expect_count_read_called);
     ck_assert(f.acceptor_handler->accepted_handler()->bytes_read() == 4);
-    f.server_io_service.run_one();
+    f.run_server_while(
+		       [&f, expect_count_read_called]() { return f.server_engine->count_read_called < expect_count_read_called + 1; });
     // The result want_write means that the previous operation
     // (in this case read) must be called once again once the
     // socket becomes writable.
-    ck_assert(f.server_engine->count_read_called == 2);
+    ck_assert(f.server_engine->count_read_called == expect_count_read_called + 1);
 }
 END_TEST
 
 START_TEST(test_read_eof)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
     f.socket->close();
     std::array<char, 1> buf;
     f.acceptor_handler->accepted_socket()->async_read(
         gu::AsioMutableBuffer(buf.data(), buf.size()),
         f.acceptor_handler->accepted_handler());
-    f.server_io_service.run_one();
+    f.run_server_while(
+        [&f]() { return f.server_engine->count_read_called < 1; });
     ck_assert(f.server_engine->count_read_called == 1);
-    ck_assert(f.acceptor_handler->accepted_handler()->last_error_code().is_eof());
+    ck_assert(
+        f.acceptor_handler->accepted_handler()->last_error_code().is_eof());
 }
 END_TEST
 
 START_TEST(test_read_error)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
     // Socket close makes the socket readable, but we override
     // the return value with error.
@@ -1671,16 +1983,18 @@ START_TEST(test_read_error)
     f.acceptor_handler->accepted_socket()->async_read(
         gu::AsioMutableBuffer(buf.data(), buf.size()),
         f.acceptor_handler->accepted_handler());
-    f.server_io_service.run_one();
+    f.run_server_while(
+        [&f]() { return f.server_engine->count_read_called < 1; });
     ck_assert(f.server_engine->count_read_called == 1);
-    ck_assert(f.acceptor_handler->accepted_handler()->last_error_code().value() == EPIPE);
+    ck_assert(f.acceptor_handler->accepted_handler()->last_error_code().value()
+              == EPIPE);
 }
 END_TEST
 
 START_TEST(test_write_want_read)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
 
     f.server_engine->next_result = gu::AsioStreamEngine::want_read;
@@ -1699,7 +2013,8 @@ START_TEST(test_write_want_read)
     ck_assert(f.socket_handler->bytes_written() == 4);
     // Now the server side socket should become readable and
     // the second call to write should happen.
-    f.server_io_service.run_one();
+    f.run_server_while(
+        [&]() { return f.server_engine->count_write_called < 2; });
     ck_assert(f.acceptor_handler->accepted_handler()->bytes_written() == 4);
     ck_assert(f.server_engine->count_write_called == 2);
 }
@@ -1708,7 +2023,7 @@ END_TEST
 START_TEST(test_write_want_write)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
 
     f.server_engine->next_result = gu::AsioStreamEngine::want_write;
@@ -1722,7 +2037,8 @@ START_TEST(test_write_want_write)
     ck_assert(f.server_engine->count_write_called == 1);
     // Now the server side socket should remain writable and the
     // the second call to write should happen.
-    f.server_io_service.run_one();
+    f.run_server_while(
+        [&f]() { return f.server_engine->count_write_called < 2; });
     ck_assert(f.acceptor_handler->accepted_handler()->bytes_written() == 4);
     ck_assert(f.server_engine->count_write_called == 2);
 }
@@ -1731,7 +2047,7 @@ END_TEST
 START_TEST(test_write_eof)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
 
     f.server_engine->next_result = gu::AsioStreamEngine::want_read;
@@ -1749,16 +2065,18 @@ START_TEST(test_write_eof)
     f.client_io_service.run_one();
     ck_assert(f.socket_handler->bytes_written() == 4);
     f.server_engine->next_result = gu::AsioStreamEngine::eof;
-    f.server_io_service.run_one();
+    f.run_server_while(
+        [&f] { return f.server_engine->count_write_called < 2; });
     ck_assert(f.server_engine->count_write_called == 2);
-    ck_assert(f.acceptor_handler->accepted_handler()->last_error_code().is_eof());
+    ck_assert(
+        f.acceptor_handler->accepted_handler()->last_error_code().is_eof());
 }
 END_TEST
 
 START_TEST(test_write_error)
 {
     TlsServiceServerTestFixture f;
-    f.server_io_service.run_one();
+    f.complete_server_handshake();
     ck_assert(f.acceptor_handler->accepted_socket() != 0);
 
     f.server_engine->next_result = gu::AsioStreamEngine::error;
@@ -1768,12 +2086,14 @@ START_TEST(test_write_error)
     cbs[1] = gu::AsioConstBuffer();
     f.acceptor_handler->accepted_socket()->async_write(
         cbs, f.acceptor_handler->accepted_handler());
-    f.server_io_service.run_one();
+    f.run_server_while(
+        [&f] { return f.server_engine->count_write_called < 1; });
     ck_assert(f.server_engine->count_write_called == 1);
     // Write will succeed before the error is injected, so there will be
     // some bytes written.
     ck_assert(f.acceptor_handler->accepted_handler()->bytes_written() == 4);
-    ck_assert(f.acceptor_handler->accepted_handler()->last_error_code().value() == EPIPE);
+    ck_assert(f.acceptor_handler->accepted_handler()->last_error_code().value()
+              == EPIPE);
 }
 END_TEST
 
@@ -1781,6 +2101,20 @@ END_TEST
 // Datagram
 //
 
+/* Helper to determine if UDP sockets can be opened. */
+static bool have_datagram() try
+{
+  gu::AsioIoService io_service;
+  gu::URI uri("udp://127.0.0.1:0");
+  auto socket(io_service.make_datagram_socket(uri));
+  socket->open(uri);
+  return true;
+}
+catch (...)
+{
+  return false;
+}
+
 class MockDatagramSocketHandler : public gu::AsioDatagramSocketHandler
 {
 public:
@@ -1865,8 +2199,7 @@ START_TEST(test_datagram_connect_multicast_local_if)
 END_TEST
 
 void test_datagram_send_to_and_async_read_common(
-    gu::AsioIoService& io_service,
-    gu::AsioDatagramSocket& socket,
+    gu::AsioIoService& io_service, gu::AsioDatagramSocket& socket,
     const std::shared_ptr<MockDatagramSocketHandler>& handler)
 {
     gu::URI local_uri(socket.local_addr());
@@ -1884,7 +2217,8 @@ void test_datagram_send_to_and_async_read_common(
                                local_uri.get_port()));
 
     char read_buf[7];
-    socket.async_read(gu::AsioMutableBuffer(read_buf, sizeof(read_buf)), handler);
+    socket.async_read(gu::AsioMutableBuffer(read_buf, sizeof(read_buf)),
+                      handler);
     while (handler->bytes_read() != sizeof(read_buf))
     {
         io_service.run_one();
@@ -1974,9 +2308,10 @@ START_TEST(test_steady_timer)
     // 50msec for some reason.
     ck_assert(
         std::chrono::duration_cast<std::chrono::milliseconds>(stop - start)
-        >= std::chrono::milliseconds(50),
+            >= std::chrono::milliseconds(50),
         "Timer duration less than 50 milliseconds %zu",
-        std::chrono::duration_cast<std::chrono::milliseconds>(stop - start).count());
+        std::chrono::duration_cast<std::chrono::milliseconds>(stop - start)
+            .count());
 #endif
 }
 END_TEST
@@ -2119,6 +2454,7 @@ Suite* gu_asio_suite()
     //
     // SSL
     //
+    generate_certificates();
 
     tc = tcase_create("test_ssl_io_service");
     tcase_add_test(tc, test_ssl_io_service);
@@ -2285,6 +2621,7 @@ Suite* gu_asio_suite()
     //
     // Datagram
     //
+    if (have_datagram()) {
 
     tc = tcase_create("test_datagram_socket");
     tcase_add_test(tc, test_datagram_socket);
@@ -2306,6 +2643,7 @@ Suite* gu_asio_suite()
     tcase_add_test(tc, test_datagram_send_to_and_async_read);
     suite_add_tcase(s, tc);
 
+    }
 #if defined(GALERA_ASIO_TEST_MULTICAST)
     tc = tcase_create("test_datagram_connect_multicast");
     tcase_add_test(tc, test_datagram_connect_multicast);
