@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Codership Oy <info@codership.com>
+// Copyright (C) 2009-2020 Codership Oy <info@codership.com>
 
 #include <vector>
 #include <deque>
@@ -30,26 +30,26 @@ START_TEST(test_resolver)
     std::string tcp_lh4("tcp://127.0.0.1:2002");
 
     Addrinfo tcp_lh4_ai(resolve(tcp_lh4));
-    fail_unless(tcp_lh4_ai.get_family() == AF_INET);
-    fail_unless(tcp_lh4_ai.get_socktype() == SOCK_STREAM);
+    ck_assert(tcp_lh4_ai.get_family() == AF_INET);
+    ck_assert(tcp_lh4_ai.get_socktype() == SOCK_STREAM);
 
-    fail_unless(tcp_lh4_ai.to_string() == tcp_lh4, "%s != %s",
-                tcp_lh4_ai.to_string().c_str(), tcp_lh4.c_str());
+    ck_assert_msg(tcp_lh4_ai.to_string() == tcp_lh4, "%s != %s",
+                  tcp_lh4_ai.to_string().c_str(), tcp_lh4.c_str());
 
     std::string tcp_lh6("tcp://[::1]:2002");
 
     Addrinfo tcp_lh6_ai(resolve(tcp_lh6));
-    fail_unless(tcp_lh6_ai.get_family() == AF_INET6);
-    fail_unless(tcp_lh6_ai.get_socktype() == SOCK_STREAM);
+    ck_assert(tcp_lh6_ai.get_family() == AF_INET6);
+    ck_assert(tcp_lh6_ai.get_socktype() == SOCK_STREAM);
 
-    fail_unless(tcp_lh6_ai.to_string() == tcp_lh6, "%s != %s",
-                tcp_lh6_ai.to_string().c_str(), tcp_lh6.c_str());
+    ck_assert_msg(tcp_lh6_ai.to_string() == tcp_lh6, "%s != %s",
+                  tcp_lh6_ai.to_string().c_str(), tcp_lh6.c_str());
 
 
     std::string lh("tcp://localhost:2002");
     Addrinfo lh_ai(resolve(lh));
-    fail_unless(lh_ai.to_string() == "tcp://127.0.0.1:2002" ||
-                lh_ai.to_string() == "tcp://[::1]:2002");
+    ck_assert(lh_ai.to_string() == "tcp://127.0.0.1:2002" ||
+              lh_ai.to_string() == "tcp://[::1]:2002");
 
 }
 END_TEST
