@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Codership Oy <info@codership.com> */
+/* Copyright (C) 2011-2024 Codership Oy <info@codership.com> */
 
 #include "garb_config.hpp"
 #include "garb_recv_loop.hpp"
@@ -10,6 +10,7 @@
 #include <stdlib.h> // exit()
 #include <unistd.h> // setsid(), chdir()
 #include <fcntl.h>  // open()
+#include <signal.h> // sigaction
 
 namespace garb
 {
@@ -100,6 +101,18 @@ main (int argc, char* argv[])
 
     try
     {
+        /* Ignore SIGPIPE which could be raised when cluster connections are
+           closed abruptly. */
+        struct sigaction isa;
+        memset (&isa, 0, sizeof(isa));
+        isa.sa_handler = SIG_IGN;
+
+        if (sigaction (SIGPIPE, &isa, NULL))
+        {
+            gu_throw_error(errno) << "Falied to install signal handler for signal "
+                                  << "SIGPIPE";
+        }
+
         RecvLoop loop (config);
         return 0;
     }
