@@ -100,13 +100,18 @@ public:
         propagate_remote_ (false),
         tp_               (tp),
         link_map_         (),
-        tstamp_           (gu::datetime::Date::now()),
+        send_tstamp_      (gu::datetime::Date::monotonic()),
+        recv_tstamp_      (gu::datetime::Date::monotonic()),
         gmcast_           (gmcast)
     { }
 
-    ~Proto() { tp_->close(); }
+    ~Proto()
+    {
+      tp_->close();
+      tp_ = nullptr;
+    }
 
-    void send_msg(const Message& msg);
+    void send_msg(const Message& msg, bool ignore_no_buffer_space);
     void send_handshake();
     void wait_handshake();
     /*
@@ -162,8 +167,10 @@ public:
         return ret;
     }
     int version() const { return version_; }
-    void set_tstamp(gu::datetime::Date ts) { tstamp_ = ts; }
-    gu::datetime::Date tstamp() const { return tstamp_; }
+    void set_recv_tstamp(gu::datetime::Date ts) { recv_tstamp_ = ts; }
+    gu::datetime::Date recv_tstamp() const { return recv_tstamp_; }
+    void set_send_tstamp(gu::datetime::Date ts) { send_tstamp_ = ts; }
+    gu::datetime::Date send_tstamp() const { return send_tstamp_; }
 private:
     friend std::ostream& operator<<(std::ostream&, const Proto&);
     Proto(const Proto&);
@@ -183,7 +190,8 @@ private:
     bool              propagate_remote_;
     SocketPtr         tp_;
     LinkMap           link_map_;
-    gu::datetime::Date tstamp_;
+    gu::datetime::Date send_tstamp_;
+    gu::datetime::Date recv_tstamp_;
     GMCast&     gmcast_;
 };
 
